<?php
// +----------------------------------------------------------------------
// | HkCms 文章模型
// +----------------------------------------------------------------------
// | Copyright (c) 2020-2021 http://www.hkcms.cn, All rights reserved.
// +----------------------------------------------------------------------
// | Author: 广州恒企教育科技有限公司 <admin@hkcms.cn>
// +----------------------------------------------------------------------

declare (strict_types=1);

namespace app\admin\model\cms;

use think\facade\Db;
use think\facade\Validate;
use think\Model;
use think\model\concern\SoftDelete;
use think\Paginator;

class Archives extends Model
{
    use SoftDelete;

    /**
     * 软删除字段
     * @var string
     */
    protected $deleteTime = 'delete_time';

    public static $tablename;

    /**
     * 指定自动时间戳类型
     * @var string
     */
    protected $autoWriteTimestamp = 'int';

    /**
     * 格式化发布日期
     * @param $value
     * @param $data
     * @return false|string
     */
    public function getPublishTimeTextAttr($value, $data)
    {
        return empty($data['publish_time']) ? '-' : date('Y-m-d H:i:s', $data['publish_time']);
    }

    /**
     * 格式化url
     * @param $value
     * @param $data
     * @return string
     */
    public function getUrlAttr($value, $data)
    {
        return $this->buildUrl($value, $data);
    }

    /**
     * 获取完整url地址
     * @param $value
     * @param $data
     * @return int|mixed|string|string[]
     */
    public function getFullurlAttr($value, $data)
    {
        return $this->buildUrl($data['url'], $data, true);
    }

    /**
     * 生成url地址
     * @param $value
     * @param $data
     * @param bool $domain
     */
    protected function buildUrl($value, $data, $domain = false)
    {
        if (empty($value)) {
            if (!empty($data['jumplink'])) {
                return $data['jumplink'];
            }
            $cateInfo = Category::where(['id'=>$data['category_id']])->find();
            if (!$cateInfo) {
                return '';
            }
            $cateInfo = $cateInfo->toArray();
            $site = site(); // 获取伪静态规则
            if ($data['status']!='normal') {
                // 去到预览页
                $param = ['category_id'=>$data['category_id'],'id'=>$data['id']];
                if ($site['content_lang_on']==1) {
                    $param['lang'] = $data['lang'];
                }
                return (string) url('/cms.archives/preview',$param);
            }
            if ($site['url_mode']==1 && !empty($site['url_rewrite'])) {
                $cateInfo['aid'] = $data['id'];
                $cateInfo['diyname'] = !empty($data['diyname']) ? $data['diyname']:'';
                $param = $cateInfo;
            } else {
                $param = ['id'=>$data['id'],'catname'=>$cateInfo['name']];
                if ($site['content_lang_on']==1) {
                    $param['lang'] = $data['lang'];
                }
            }
            return index_url($cateInfo['type']=='link'?'/index/lists':'/index/show', $param, '', $domain);
        } else if ($domain) {
            if (!\think\facade\Validate::regex($value, '/^https?:\/\/(([a-zA-Z0-9_-])+(\.)?)*(:\d+)?\//i')) {
                $value = app('request')->domain() . $value;
            }
        }
        return $value;
    }

    /**
     * 格式化
     * @param $value
     * @param $data
     * @return false|string
     */
    public function getDeleteTimeTextAttr($value, $data)
    {
        return empty($data['delete_time']) ? '-' : date('Y-m-d H:i:s', $data['delete_time']);
    }

    /**
     * save 方法下，新增，修改都会触发
     * @param Model $model
     * @return mixed|void
     */
    public static function onBeforeWrite($model)
    {
        $publishTime = $model->getData('publish_time');
        if (empty($publishTime)) {
            $model->setAttr('publish_time', time());
        } else if (!is_numeric($publishTime)) {
            $model->setAttr('publish_time', strtotime($publishTime));
        }

        $data = $model->getData();
        if (isset($data['flags'])) {
            $flags = explode(',', $data['flags']);
            if (in_array('top', $flags)) {
                $model->setAttr('weigh', 1);
            } else if ($data['weigh']==1) {
                $model->setAttr('weigh', 100);
            }
        }
    }

    /**
     * 查询后，使用与模型->select()/find()后执行save，保证表名称保持一致。
     * @param Model $model
     */
    public static function onAfterRead(Model $model)
    {
        if (!empty(self::$tablename)) {
            $model->name = $model->getName() != self::$tablename ? self::$tablename:$model->getName();
        }
    }

    /**
     * 获取栏目对应的表格信息
     * @param $categoryId
     * @param $categoryInfo
     * @return array|Model|null
     */
    public function getTableInfo($categoryId, &$categoryInfo=null)
    {
        if (empty($categoryId)) {
            abort(404, lang('%s not null',['category_id']));
        }
        $categoryInfo = Category::where(['id'=>$categoryId,'status'=>'normal'])->find();
        if (empty($categoryInfo)) {
            abort(404, lang('Column information does not exist'));
        }
        $modelInfo = \app\admin\model\cms\Model::where(['status'=>'normal', 'id'=>$categoryInfo['model_id']])->find();
        if (empty($modelInfo)) {
            abort(404, lang('Model information does not exist'));
        }
        return $modelInfo;
    }

    /**
     * 无需表前缀, 定义表名称
     * @param $name
     * @return $this
     */
    public function setTable($name)
    {
        $this->name = $name;
        self::$tablename = $name;
        return $this;
    }

    /**
     * 固定方法，用于栏目删除后的处理
     * @param $modelInfo
     * @param $categoryId
     * @param int $force 1=分类放入回收站，2=还原，3=销毁
     */
    public function handleDel($modelInfo, $categoryId, $force=1)
    {
        if ($force==3) {
            $ids = \think\facade\Db::name('archives')->where(['category_id'=>$categoryId])->where('delete_time','not null')->column('id');
            \think\facade\Db::name('archives')->where(['category_id'=>$categoryId])->where('delete_time','not null')->delete();
            \think\facade\Db::name($modelInfo['tablename'])->whereIn('id',$ids)->delete();
        } else if ($force == 2) {
            (new Archives)->onlyTrashed()->where(['category_id'=>$categoryId])->update(['delete_time'=>null]);
        } else {
            (new Archives)->where(['category_id'=>$categoryId])->update(['delete_time'=>time()]);
        }
    }

    /**
     * 获取列表，用于内容标签
     * @param $tag
     * @param $catData
     * @param $modelData
     * @param $page \think\Paginator|mixed paginate 分页对象
     * @return array|mixed|object|\think\App
     */
    public function tagContent($tag, $catData, $modelData, &$page)
    {
        $tag['order'] = !empty($tag['order']) ? $tag['order'] : 'weigh asc,id desc';
        $tag['flag'] = !empty($tag['flag']) ? $tag['flag'] : '';

        // 语言切换
        $tag['lang'] = site('content_lang');
        if (site('content_lang_on')==1) {
            $tag['lang'] = app()->lang->getLangSet();
        }

        // 增加缓存非分页数据
        if (!$tag['page']) {
            $cacheID = to_guid_string($tag);
            if (!env('APP_DEBUG') && $cacheData = cache($cacheID)) {
                return $cacheData;
            }
        }

        // 默认条件
        $map = [['status','=','normal'],['lang','=',$tag['lang']]];

        // 检查条件字段是否加了id
        if (empty($tag['aid']) && empty($tag['aids'])) {
            if (is_string($tag['where'])) {
                $sortArr = explode(' ',$tag['where']);
                foreach ($sortArr as $key=>$item) {
                    $sortArr[$key] = stripos($item, "id")===0 ? 'a.'.$item : $item;
                }
                $tag['where'] = implode(' ', $sortArr);
            } else if (is_array($tag['where'])) {
                foreach ($tag['where'] as $key=>$value) {
                    if ($key=='id') {
                        $tag['where']['a.id'] = $value;
                        unset($tag['where']['id']);
                        break;
                    }
                }
            }
        }

        $obj = $this->alias('a')->with(['category','model'])->where($tag['where']);
        // 文档属性
        if (!empty($tag['flag'])) {
            $sql = [];
            if (stripos($tag['flag'], ' and ')) {
                $flag = explode(' and ', $tag['flag']);
                foreach ($flag as $key=>$val) {
                    $sql[] = "find_in_set('{$val}',flags)";
                }
                if ($sql) {
                    $sql = implode(' and ', $sql);
                }
            } else if (stripos($tag['flag'], ' or ')) {
                $flag = explode(' or ', $tag['flag']);
                foreach ($flag as $key=>$val) {
                    $sql[] = "find_in_set('{$val}',flags)";
                }
                if ($sql) {
                    $sql = "(".implode(' or ', $sql).")";
                }
            } else {
                $sql = "find_in_set('{$tag['flag']}',flags)";
            }
            $obj = $obj->where($sql);
        }

        if ($tag['aid']) { // 指定单条文档
            $obj = $obj
                ->where($map)
                ->where('id','=',$tag['aid'])
                ->order($tag['order'])
                ->append(['publish_time_text','fullurl'])
                ->select()
                ->toArray();
            if (empty($obj)) {
                return [];
            }
            $array = $obj[0];
            $modelInfo = \app\admin\model\cms\Model::where(['status'=>'normal','id'=>$array['model']['id']])->cache(env('APP_DEBUG')?false:3600)->find();
            if (empty($modelInfo)) {
                return [];
            }
            $info = Db::name($modelInfo['tablename'])->find($array['id']);
            $array = array_merge($array, empty($info)?[]:$info);
            // 获取扩展字段
            $fields = \think\facade\Db::name('model_field')->where(['status'=>'normal','model_id'=>$array['model']['id']])->cache(env('APP_DEBUG')?false:3600)->select()->toArray();
            $array = [$array];
        } else if ($tag['aids']) { // 指定多条文档
            $obj = $obj
                ->where($map)
                ->whereIn('id', $tag['aids'])
                ->order($tag['order'])
                ->append(['publish_time_text','fullurl'])
                ->select()
                ->toArray();
            if (empty($obj)) {
                return [];
            }

            $array = $obj[0];
            $modelInfo = \app\admin\model\cms\Model::where(['status'=>'normal','id'=>$array['model']['id']])->cache(env('APP_DEBUG')?false:3600)->find();
            if (empty($modelInfo)) {
                return [];
            }

            $newArray = [];
            foreach ($obj as $key=>$value) {
                $info = Db::name($modelInfo['tablename'])->find($value['id']);
                $array = array_merge($value, empty($info)?[]:$info);
                $newArray[] = $array;
            }

            // 获取扩展字段
            $fields = \think\facade\Db::name('model_field')->where(['status'=>'normal','model_id'=>$array['model']['id']])->cache(env('APP_DEBUG')?false:3600)->select()->toArray();
            $array = $newArray;
        } else {
            // 查出模型数据
//            if (empty($modelData)) {
//                // 未指定模型使用栏目来查数据
//                if (empty($tag['model']) && !empty($catData)) {
//                    $tag['model'] = isset($catData[0]['model_id'])?$catData[0]['model_id']:$catData['model_id'];
//                } else if (empty($tag['model']) && $tag['catid']) {
//                    $catid = explode(',',(string)$tag['catid']);
//                    $cateInfo = Category::where(['id'=>$catid[0],'status'=>'normal','lang'=>$tag['lang']])->cache(env('APP_DEBUG')?false:3600)->find();
//                    if (empty($cateInfo['model_id'])) {
//                        return [];
//                    }
//                    $tag['model'] = $cateInfo['model_id'];
//
//                }
//                $modelData = \app\admin\model\cms\Model::where(['status'=>'normal','id'=>$tag['model']])->cache()->find();
//                if (empty($modelData)) {
//                    return [];
//                }
//            }

            $tag['model'] = empty($tag['model']) ? $modelData['id']??'':$tag['model'];
            $tag['catid'] = empty($tag['catid']) ? $catData['id']??'':$tag['catid'];
            $map[] = ['model_id','=',$modelData['id']];

            // 获取请求的排序、数据筛选
            $sort = app()->request->get('sort');
            $order = app()->request->get('order');
            if ($sort!='_default' && Validate::is($sort, 'alphaDash')) {
                $mf = ModelField::where(['field_name'=>$sort])->cache()->find();
                if (!empty($mf)) {
                    $tag['order'] = [($mf->iscore==1?'':'x.').$sort=>$order=='desc'?$order:'asc'];
                }
            }
            
            $param = app()->request->get();
            $filter = array_diff_key($param, array_flip(['sort','order','lang']));
            $filterField = ModelField::where(['model_id'=>$tag['model'],'is_filter'=>1])->cache(true)->select()->toArray();
            foreach ($filterField as $key=>$value) {
                if (isset($filter[$value['field_name']]) && $filter[$value['field_name']]!='') {
                    $tempArr = explode(',', $filter[$value['field_name']]);
                    // 匹配是否是筛选的值
                    $setting = json_decode($value['setting'], true);
                    if (!empty($setting['filter_option'])) {
                        $option = json_decode($setting['filter_option'], true);
                    } else if (!empty($value['data_list'])) {
                        $option = $value['data_list'];
                    }
                    if (!empty($option)) {
                        $option = array_flip($option);
                        foreach ($tempArr as $k=>$v) {
                            if (!in_array($v,$option)) {
                                unset($tempArr[$k]);
                            }
                        }
                        if (count($tempArr)>1) {
                            $map[] = [($value['iscore']!=1?'x.':'').$value['field_name'],'in',implode(',',$tempArr)];
                            $param = array_merge($param, [$value['field_name']=>implode(',',$tempArr)]);
                        } else if (count($tempArr)==1) {
                            $map[] = [($value['iscore']!=1?'x.':'').$value['field_name'],'=',$tempArr[0]];
                            $param = array_merge($param, [$value['field_name']=>$tempArr[0]]);
                        }
                    }
                }
            }

//            if ($modelData['controller']=='Single' && (!isset($tag['inlist']) || $tag['inlist']==0)) {
//                $tag['insub'] = 0;
//            }

            // 获取栏目ID，包含下级。
            if (!empty($tag['catid']) && is_numeric($tag['catid']) && $tag['insub']==1) {
                $catIdArr = get_category_sub($tag['catid'], false, ['status'=>'normal','model_id'=>$tag['model']]);
                if (!empty($catIdArr)) {
                    $catIdArr[] = $tag['catid'];
                    $obj = $obj->where(function ($query) use($tag, $catIdArr) {
                        $query->where('category_id','in', $catIdArr)->whereOr('find_in_set("'.$tag['catid'].'",category_ids)');
                    });
                } else {
                    $obj = $obj->where('(category_id = '.$tag['catid'].' or find_in_set("'.$tag['catid'].'",category_ids))');
                }
            } else if (!empty($tag['catid']) && is_numeric($tag['catid']) && $tag['insub']==0) {
                // 栏目与副栏目
                $obj = $obj->where('(category_id = '.$tag['catid'].' or find_in_set("'.$tag['catid'].'",category_ids))');
            } else if (!empty($tag['catid'])) { // 同时获取多个栏目的数据
                $obj = $obj->where(function ($query) use($tag) {
                    $query->where('category_id','in', $tag['catid']);
                    $catIdArr = explode(',',(string)$tag['catid']);
                    foreach ($catIdArr as $key=>$value) {
                        $query->whereOr('find_in_set("'.$value.'",category_ids)');
                    }
                });
            }

            // 获取扩展字段
            $fields = \think\facade\Db::name('model_field')->where(['status'=>'normal','model_id'=>$modelData->id])->cache()->select()->toArray();
            $extField = [];
            foreach ($fields as $key=>$value) {
                if ($value['iscore']==0) {
                    $extField[] = $value['field_name'];
                }
            }
            $extField = implode(',',$extField);

            if ($tag['page'] && empty($tag['tagid'])) { // 开启分页
                $obj = $obj
                    ->join($modelData->tablename.' x','a.id=x.id','LEFT')
                    ->where($map)
                    ->order($tag['order'])
                    ->field('a.*,x.id as xid,'.$extField)
                    ->append(['publish_time_text','fullurl'])
                    ->paginate([
                        'list_rows'=> intval($tag['num']),
                        'var_page' => 'page',
                        'query' => $param,
                        'path'=>'/index/lists'
                    ], false);

                $array = $obj->toArray()['data'];
                $page = $obj;
            } else if ($tag['page'] && !empty($tag['tagid'])) { // 瀑布流分页
                $page = Paginator::getCurrentPage('page');
                $obj = $obj->join($modelData->tablename.' x','a.id=x.id','LEFT')
                    ->where($map)
                    ->order($tag['order'])
                    ->field('a.*,x.id as xid,'.$extField)
                    ->append(['publish_time_text','fullurl'])
                    ->page($page, intval($tag['num']))
                    ->select();
                $array = $obj->toArray();

                $tag['key'] = md5($tag["tagid"]);
                $total = cache($tag['key']);
                if (empty($total['total'])) {
                    // 数量
                    $total = $obj->count();
                } else {
                    $total = $total['total'];
                }
                $tag['total'] = $total;
                $page = $tag;
                $page['map_param'] = $param;
                cache($tag['key'],$page);
            } else { // 不分页
                // 限制结果
                $offset = 0;
                $length = null;
                if (!empty($tag['num']) && is_numeric($tag['num']) && $tag['num']>0) { // 指定分页
                    $offset = intval($tag['num']);
                } else if (!empty($tag['num']) && strpos($tag['num'], ',') !== false) {
                    $temp = explode(',', $tag['num']);
                    if (count($temp)==2 && is_numeric($temp[0]) && is_numeric($temp[1])) {
                        $offset = (int)$temp[0]-1;
                        $length = (int)$temp[1];
                    }
                }

                $array = $obj
                    ->join($modelData->tablename.' x','a.id=x.id','LEFT')
                    ->where($map)
                    ->order($tag['order'])
                    ->limit($offset,$length)
                    ->field('a.*,x.id as xid,'.$extField)
                    ->append(['publish_time_text','fullurl'])
                    ->select()
                    ->toArray();
            }
        }

        // 字段格式化
        foreach ($array as $key=>$value) {
            foreach ($fields as $k=>$v) {
                field_format($v, $array[$key]);
                // 兼容旧版模板
                $array[$key]['more'][$v['field_name']] = $array[$key][$v['field_name']];
            }
        }

        if (!$tag['page']) {
            // 结果进行缓存
            if (!env('APP_DEBUG')) {
                // 缓存设置
                $cacheTime = !empty($tag['cache']) && is_numeric($tag['cache']) ? intval($tag['cache']) : 3600;
                cache($cacheID, $array, $cacheTime);
            }
        }
        return $array;
    }

    /**
     * 用于获取单条标签
     * @param $tag
     * @param $model
     * @return array
     */
    public function tagArcone($tag, $model)
    {
        if (empty($tag['aid']) || !is_numeric($tag['aid'])) {
            return [];
        }

        $cacheTime = !empty($tag['cache']) && is_numeric($tag['cache']) ? intval($tag['cache']) : 3600;

        if (!empty($tag['more']) && $tag['more']==1 && $model['type']=='more' && $model['allow_single']!=1) { // 副表
            $info = $this->alias('a')->with(['category','model'])
                ->join($model->tablename.' x','a.id=x.id','LEFT')
                ->where('x.id',$tag['aid']);
        } else {
            $info = $this->with(['category','model'])
                ->where('id',$tag['aid']);
        }

        if (!env('APP_DEBUG')) {
            $info = $info->cache($cacheTime);
        }

        $info = $info->append(['publish_time_text','url'])->find();
        if (empty($info)) {
            return [];
        }

        $array = [$info->toArray()];
        // 获取扩展字段
        $fields = \think\facade\Db::name('model_field')->where(['status'=>'normal','model_id'=>$model->id])->cache()->select()->toArray();
        // 字段格式化
        foreach ($array as $key=>$value) {
            foreach ($fields as $k=>$v) {
                field_format($v, $array[$key]);
                // 兼容旧版
                if ($v['iscore']!=1 && !empty($tag['more']) && $tag['more']==1 && $model['type']=='more' && $model['allow_single']!=1) {
                    $array[$key]['more'][$v['field_name']] = $array[$key][$v['field_name']];
                }
            }
        }

        return $array;
    }

    /**
     * 栏目相对关联
     * @return \think\model\relation\BelongsTo
     */
    public function category()
    {
        return $this->belongsTo(Category::class);
    }

    /**
     * 模型相对关联
     * @return Archives|\think\model\relation\BelongsTo
     */
    public function model()
    {
        return $this->belongsTo(\app\admin\model\cms\Model::class);
    }

    /**
     * 关联副表
     * @return \think\model\relation\HasOne
     */
//    public function moreInfo()
//    {
//        \app\admin\model\cms\ArchivesX::$tablename = self::$tablename;
//        return $this->hasOne(\app\admin\model\cms\ArchivesX::class,'id','id');
//    }
}